"use strict";
this.name        = "deep_space_pirates"; 
this.author      = "eric walch"; 
this.copyright   = "� 2008-2014 Creative Commons: attribution, non-commercial, sharealike."; 
this.description = "This script places random ships in deep space."; 
this.version     = "1.5";
/*
The script does nothing when in interstellar space or when docked or when under attack.
The chance of encounters is less when traveling slow. NB jumpSpeed is about 32 * maxSpeed.
It only add ships when not within "this.spacelaneRadius" meters from the core of: the spacelane planet-witchpoint, the spacelane planet-sun or any station.
Decrease change of encounters logaritmic with distance.
*/

this.logging = false;
this.spacelaneRadius = 100E3;
this.addition = 0;
 // planet radius is between 2816 km and 6912 km on F7 screen, main-station at twice this distance.
 // In calculation Oolite uses internal 28160 til 69120 meters

this.startUp = function ()
{
	if (!missionVariables.deep_space_pirate_kills) missionVariables.deep_space_pirate_kills = 1; // 0 would be a problem with Math.log()
    this.additiontime = clock.seconds;
    // With the long range scanner available, also pirates can find the player better.
    if (player.ship.equipmentStatus("EQ_LONG_RANGE_SCANNER") != "EQUIPMENT_UNKNOWN") this.addition = 10;
    
    this.dsGroup = new ShipGroup("DeepSpaceShips"); // start with an empty group
}

this.shipLaunchedFromStation = function ()
{
    if (this.checkTimer)
        this.checkTimer.start()
    else
        this.checkTimer = new Timer(this, this.dsCheck, 30, 10)
}

this.dsCheck = function ()
{
    if (!system.isInterstellarSpace 
            && !player.ship.docked
            && player.ship.speed > 20 * player.ship.maxSpeed * Math.random() 
            && !player.alertHostiles 
            && clock.seconds - this.additiontime > 120) 
    {
        this.dsAddPirate();
    }
    
    this.dsRemoveShips();
}

this.dsAddPirate = function ()
{
    // first check planet-witchpoint spacelane cylinder.
    var playerPosition = player.ship.position;
    var distanceToSpacelane = Math.sqrt(playerPosition.x * playerPosition.x + playerPosition.y * playerPosition.y);
    var chanceFactor = 0.2 / Math.log(distanceToSpacelane / this.spacelaneRadius); // value is nagative up to this.spacelaneRadius.

    // Above this.spacelaneRadius meter distance, chanceFactor starts high but decays gradually.
    if (distanceToSpacelane < 3*this.spacelaneRadius) chanceFactor *= 3 - system.government/3.5; // varies between 1 and 3
    if (chanceFactor < Math.random()) return; // stop further calculations.

    // calculate planet-sun spacelane cylinder.
    var sunSpacelane = new Vector3D(system.mainPlanet.position.subtract(system.sun.position));
    var sunPlayerVector = new Vector3D(playerPosition.subtract(system.sun.position));
    const distanceToSun = sunPlayerVector.magnitude();
    const distanceToSunSpacelane = distanceToSun * Math.sin(sunSpacelane.angleTo(sunPlayerVector));

    if (distanceToSpacelane > this.spacelaneRadius && distanceToSunSpacelane > this.spacelaneRadius)
    {
        var heading = player.ship.heading; 
        var forwardPosition = new Vector3D();
        if (Math.random()  <  0.6 - system.government / 20)  // 60% chance of pirate in Anarchie, 25% in Corporate State
        {
            if (system.countShipsWithRole("station", player.ship, 100E3) > 0) return;
            // no Pirate addition when > 150 or when within 100 000 m of station
            forwardPosition = playerPosition.add(heading.multiply(50E3));
			// var maxShipNumber = 10.5 + Math.log(missionVariables.deep_space_pirate_kills) - system.government/2
			 // Base max is 7, in anarchies up to 3.5 ships extra and 3.2 extra for every decade of kills.
			const maxShipNumber = Math.log(++this.addition) + Math.log(missionVariables.deep_space_pirate_kills) / 2 + 5.5 - system.government / 2;
			 // Base max is 2, in anarchies up to 3.5 ships extra and 3.2 extra for every decade of kills
			 // + 3.2 extra for every decade of addition during this startup.
			if (this.addition>100) this.addition = 10; // don't let the value grow to large. 

            var pirateGroup = system.addGroup("pirate", this.dsBiasedRandom(maxShipNumber), forwardPosition, 5E3);
            var pirates = pirateGroup.ships;
            pirateGroup.name = "Deep Space Pirates";
            this.dsAddShipsToQueue(pirateGroup);
        
            for (var i=0; i<pirates.length;i++)
            {
                if (pirates[i].escortGroup && pirates[i].escortGroup.count>1)
                {
                    for (var j=0; j<pirates[i].escortGroup.count;j++)
                    {
                        var escort = pirates[i].escortGroup.ships[j];
                        
                        if (escort !== pirates[i])
                        {
                            escort.switchAI("DSPescortAI.plist");
                            escort.script.shipDied = this.dsPirateDied;
                        }
                    };
                }

                if (pirates[i].cargoSpaceCapacity > 15 && pirates[i].entityPersonality&7<3) 
                {
                    pirates[i].bounty = 0;
                    pirates[i].script.shipDied = this.dsPirateDiedWithCargo;
                }
                else 
                {
                    pirates[i].script.shipDied = this.dsPirateDied;
                }
                if (pirates[i].fuel  ===  0) pirates[i].fuel = 7;
            }
            
            // Convert the script of the first ship.
            // Check if it has an old PirateAI.plist or one of the newer "oolite-piratexxxx.js" scripts. Only than change its AI behaviour
            if (pirates[0].AI  ===  "pirateAI.plist" || (pirates[0].AI  === "nullAI.plist" && pirates[0].AIScript.name.indexOf("Oolite Pirate") > -1))
            {
                var state = (distanceToSpacelane < 800E3) ? "SPACELANE" : "DEEPSPACE";
                if (pirates.length>10) state = "HIDEOUT";
                pirates[0].switchAI("DSPpirateAI.plist");
                pirates[0].AIState = state;
            }

            if (this.logging) log("DeepSpacePirate", "Added "+ pirates.length +" pirates out of a possible "+ Math.ceil(maxShipNumber) + ". Killcounter: " + missionVariables.deep_space_pirate_kills);
            this.additiontime = clock.seconds;
        }
        else
        {
            if (system.countShipsWithRole("station", player.ship, 250E3) > 0) return;
            // no when within 250 000 m of station
            
            var ship;
            var chance = Math.random();
            var count = 1;
            
            if (chance < 0.10)
            {
                ship = "trader";
            }
            else if (chance < 0.20)
            {
                ship = "trader-courier";
            }
            else if (chance < 0.30)
            {
                ship = "trader-smuggler";
            }
            else if (chance < 0.40)
            {
                ship = "hunter";
            }
            else if (chance < 0.50)
            {
                ship = system.techLevel < 8 ? "police" : "interceptor";
            }
            else if (chance < 0.99)
            {
                ship = "asteroid";
                count = this.dsBiasedRandom(6);
            }
            else
            {
                ship = "RockHermite";
            };
            
            if (this.logging) log("DeepSpacePirate", "Adding " + ship);
            if (ship != "RockHermite")
            {
                forwardPosition = playerPosition.add(heading.multiply(90E3));
                this.dsAddShipsToQueue(system.addGroup(ship, count, forwardPosition, 10E3));
            }
            else
            {
                forwardPosition = playerPosition.add(heading.multiply(200E3));
                this.dsAddShipsToQueue(system.addGroup("DSProckhermit", 1, forwardPosition, 10E3));
                this.dsAddShipsToQueue(system.addGroup("asteroid", 8, forwardPosition, 10E3));
                // The only reason for using a custom RockHermite is that we had a yellow scanscass in the past
                // when adding the build-in one. With Oolite 1.80 this is probably no longer needed.
            }
        }
    }
}

this.dsAddShipsToQueue = function (group)
{
    if (group && group.count > 0)
    {
        for (var i=0; i<group.count;i++)
        {
            this.dsGroup.addShip(group.ships[i]);
            
            if (group.ships[i].escortGroup.count > 1)
            {
                for (var j=0; j<group.ships[i].escortGroup.count;j++)
                {
                    this.dsGroup.addShip(group.ships[i].escortGroup.ships[j]);
                }
            }
        }
    }
    if (this.logging) log("DeepSpacePirate", "Shipgroup: " + this.dsGroup.ships);
}

this.dsRemoveShips = function ()
{
    for (var i=0; i<this.dsGroup.count;i++)
    {
        if (!this.dsGroup.ships[i].isVisible) 
        {
            this.dsGroup.ships[i].remove(true);
            if (this.logging) log("DeepSpacePirate", "removing: " + this.dsGroup.ships[i].primaryRole);
        }
    }
}

this.dsBiasedRandom = function (amount)
{
    return Math.ceil(Math.pow(Math.random(), 3) * amount);
}

// next code is used by ship scripts.
this.dsPirateDied = function (whom)
{
	if (whom && whom.isPlayer && !player.ship.isCloaked) missionVariables.deep_space_pirate_kills++;
}

this.dsPirateDiedWithCargo = function (whom)
{
	if (whom && whom.isPlayer && !player.ship.isCloaked) missionVariables.deep_space_pirate_kills++;
	var cargo = Math.ceil(10 * Math.random());
	if (this.ship.cargoSpaceAvailable < 8) cargo = this.ship.cargoSpaceAvailable;
	if (cargo > 0) this.ship.spawn("cargopod", cargo);
}

this.dsShipAcceptedEscort = function (escort)
{
		if (escort.script.asigned) return; // escort is already initialised by another ds mother that now has died.
		if (escort.bounty === 0) escort.bounty = this.ship.bounty + 5;
		escort.switchAI("DSPescortAI.plist");
		escort.script.shipDied = worldScripts["deep_space_pirates"].dsPirateDied;
		escort.script.asigned = true;
		if (this.logging) log(this.name, this.ship.displayName+" accepted escort");
}